%This is the main file of the 2D Laplace's finite difference solver. Here
%is a summary of what each function does

%f_bound: this computes the boundary condition value of the problem
%f_analy: this computes the analytical solution
%local_to_global: this converts the local (coordinate) index to global
%index
%global_to_local: this converts the global index to local (coordinate)
%index

%Your task is to fill in two lines

%(1) Solve Au=b in this file
%(2) Fill in the analytical solution in the file f_analy.m


N_tot = 50; 
error_array = zeros(N_tot-1,1);

a = 0;
b = 1; %The domain is [a,b]-by-[a,b]
for N=2:N_tot
    %N = 10; %N refers to the number of inner nodes. So the grid should be N by N 

    A = zeros(N * N, N * N);
    h = (b - a) / (N + 1);
    b_vec = zeros(N*N,1);

    x_array = linspace(a, b, N+2);
    x_array = x_array(2:N+1);

    y_array = linspace(a, b, N+2);
    y_array = y_array(2:N+1);


    [X,Y] = meshgrid(x_array, y_array);
    
    for l=1:N*N % Matrix assembly
        indices = global_to_local(l, N); %convert global to coordinate index
        i_index = indices(1);
        j_index = indices(2);

        A(l,l) = 4.;
         if i_index > 1
             A(l, local_to_global(i_index - 1,j_index, N)) = -1;

         else
            b_vec(l) = b_vec(l) + f_bound(a + (i_index - 1) * h, a + (j_index) * h)  ;
         end
    %     
         if i_index < N
              A(l, local_to_global(i_index + 1,j_index, N)) = -1;

         else
             b_vec(l) = b_vec(l) + f_bound(a + (i_index + 1) * h, a + (j_index) * h) ;
         end
    %     
         if j_index > 1
             A(l, local_to_global(i_index,j_index - 1, N)) = -1;

         else
             b_vec(l) = b_vec(l) + f_bound(a + (i_index) * h, a + (j_index - 1) * h) ;
         end
    %     
         if j_index < N
             A(l, local_to_global(i_index,j_index + 1, N)) = -1;

         else
             b_vec(l) = b_vec(l) + f_bound(a + (i_index) * h, a + (j_index + 1) * h)  ;

         end
    end

    A = A/h^2;
    b_vec = b_vec / h^2;
    u_vec = %Solve for A u_vec = b_vec% 
    
    u_array = reshape(u_vec, N, N)';
    u_analy = f_analy(X,Y);

    error_array(N-1,1) =  (norm(abs(u_analy - u_array),2) * h);
   
end

 
figure(1)
imagesc(x_array, y_array, u_array);

figure(2)
fit_params = polyfit(log(2:1:N_tot),log(error_array'),1); %linear fit on log-log scale
slope = fit_params(1);
y_int = fit_params(2);
loglog(2:1:N_tot, error_array, 'o')


loglog((2:1:N_tot),error_array ,'o')
hold on
loglog((2:1:N_tot), (2:1:N_tot).^slope * exp(y_int),'r')
title("L2 Norm Decay of the Solution")
xlabel("N")
ylabel("L^2 Error")
txt = ['Slope = ' num2str(slope)  ];
text(N_tot/2,error_array(N_tot/2),txt,'FontSize',14)
